#include <config.h>
#include <gtk/gtk.h>
#include <glade/glade-xml.h>
#include "hawksmoor.h"

void hm_dump_info ( HM_Inst *hm ) {

	g_print ("Filename: %s\n",hm->filename);
	// g_print ("Pixbuf: %d\n",hm->pixbuf);
	g_print ("Change Flag: %s\n", hm->changed ? "true" : "false");
}

gboolean hm_load_image ( HM_Inst *hm, gchar *filename ) {

	gchar		*title;
	GError		*error = NULL;
	
	/* try to load the pixbuf */
	hm->pixbuf = gdk_pixbuf_new_from_file (filename, &error);
	if (error) return FALSE;
	gtk_image_set_from_pixbuf (GTK_IMAGE(hm->image), hm->pixbuf);

	/* set the window title to reflect loaded image */
	title = g_strdup_printf ("Hawksmoor : %s", filename);
	gtk_window_set_title (GTK_WINDOW(hm->window), title);
	g_free (title);
	g_free (error);

	hm->width = gdk_pixbuf_get_width (hm->pixbuf);
	hm->height = gdk_pixbuf_get_height (hm->pixbuf);
	hm->filename = g_strdup (filename);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(hm->h_spin), hm->height);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(hm->w_spin), hm->width);
	hm->changed = FALSE;
	return TRUE;
}

gboolean hm_resize_image (GtkWidget *widget, HM_Inst *hm ) {

	gint height, width;
	height = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(hm->h_spin));
	width = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(hm->w_spin));	
	hm->pixbuf = gdk_pixbuf_scale_simple ((const GdkPixbuf *)hm->pixbuf, width, height, GDK_INTERP_BILINEAR);
	gtk_image_set_from_pixbuf(GTK_IMAGE(hm->image),hm->pixbuf);
	hm->changed = TRUE;
	return TRUE;
}

void hm_save_image ( GtkWidget *widget, HM_Inst *hm ) {

        GtkWidget       *dialog;
        gint            response;
	gchar		*type;
	GError		*error = NULL;

	g_return_if_fail (hm->pixbuf);
        dialog = gtk_file_selection_new ("Hawksmoor : Save Image");
        gtk_file_selection_set_filename (GTK_FILE_SELECTION(dialog), hm->filename);
        response = gtk_dialog_run (GTK_DIALOG(dialog));
	if (response == GTK_RESPONSE_CANCEL) return;
	hm->filename = g_strdup (gtk_file_selection_get_filename(GTK_FILE_SELECTION(dialog)));
	gtk_widget_destroy (dialog);
	if (g_str_has_suffix(hm->filename,"jpg")) {
		type = "jpeg";
	} else {
		type = "png";
	}
	gdk_pixbuf_save (hm->pixbuf,hm->filename,type,&error,NULL);
	if (error) return;
	hm->changed = FALSE;
	g_free (type);
}

void hm_open_image ( GtkWidget *widget, HM_Inst *hm ) {

        GtkWidget       *dialog;
        gint            response;
	gchar		*filename;
        GError          *error;

        dialog = gtk_file_selection_new ("Hawksmoor : Open Image");
        response = gtk_dialog_run (GTK_DIALOG(dialog));
        if (response == GTK_RESPONSE_CANCEL) return;
	filename = g_strdup (gtk_file_selection_get_filename(GTK_FILE_SELECTION(dialog)));
	gtk_widget_destroy (dialog);
	hm_load_image (hm, filename);
	return;
}

gboolean hm_app_quit (GtkWidget *widget, HM_Inst *hm ) {

	if (hm->changed) {
		GtkWidget	*dialog;
		gint		response;

		dialog = gtk_message_dialog_new ( NULL, 0, GTK_MESSAGE_QUESTION,
						  GTK_BUTTONS_YES_NO,
						  "<b>%s</b>\n\n%s",
						  "The file has been modified",
						  "Do you want to save it?");
		gtk_label_set_use_markup (GTK_LABEL(GTK_MESSAGE_DIALOG(dialog)->label), TRUE);
		response = gtk_dialog_run (GTK_DIALOG(dialog));
		if (response == GTK_RESPONSE_YES) hm_save_image (hm->save, hm);
                gtk_widget_destroy (dialog);

	}

	gtk_main_quit ();
	exit (0);
	
}

int main ( int argc, char *argv[] ) {

	GladeXML	*hm_glade;
	GtkWidget	*hm_main_window;
	HM_Inst		*hm;

	hm = g_new0 (HM_Inst,1);

	gtk_init (&argc, &argv);
	hm_glade = glade_xml_new (PACKAGE_DATA_DIR "/hawksmoor/hawksmoor.glade",
				  NULL, NULL);
	if (hm_glade == NULL) exit (1);
	hm->window = glade_xml_get_widget (hm_glade, "hm_main_window");
	hm->image = glade_xml_get_widget (hm_glade, "hm_image");
	hm->h_spin = glade_xml_get_widget (hm_glade, "hm_height");
	hm->w_spin = glade_xml_get_widget (hm_glade, "hm_width");
	hm->quit = glade_xml_get_widget (hm_glade, "hm_quit");
	hm->open = glade_xml_get_widget (hm_glade, "hm_open");
	hm->save = glade_xml_get_widget (hm_glade, "hm_save");
	gtk_spin_button_set_increments (GTK_SPIN_BUTTON(hm->h_spin),10,50);
	gtk_spin_button_set_increments (GTK_SPIN_BUTTON(hm->w_spin),10,50);

	if (argc > 1) {
		hm_load_image (hm, argv[1]);
	} else {
		hm_open_image (hm->save, hm);
	}

	if (hm->pixbuf == NULL) exit (1);

	g_signal_connect (G_OBJECT(hm->window),"delete_event",G_CALLBACK(hm_app_quit), hm);
	g_signal_connect (G_OBJECT(hm->window),"destroy_event",G_CALLBACK(hm_app_quit), hm);
	g_signal_connect (G_OBJECT(hm->h_spin),"value_changed",G_CALLBACK(hm_resize_image), hm);
	g_signal_connect (G_OBJECT(hm->w_spin),"value_changed",G_CALLBACK(hm_resize_image), hm);
	g_signal_connect (G_OBJECT(hm->quit),"clicked",G_CALLBACK(hm_app_quit), hm);
	g_signal_connect (G_OBJECT(hm->open),"clicked",G_CALLBACK(hm_open_image), hm);
	g_signal_connect (G_OBJECT(hm->save),"clicked",G_CALLBACK(hm_save_image), hm);
	gtk_widget_show_all (hm->window);
	gtk_main();

	
}
